@@ -32,6 +32,7 @@ class View(FreezableMmifObject):
     """
 
     def __init__(self, view_obj: Union[bytes, str, dict] = None) -> None:
+        self._id_counts = {}
         self.id: str = ''
         self.metadata: ViewMetadata = ViewMetadata()
         self.annotations: AnnotationsList = AnnotationsList()
@@ -59,7 +60,7 @@ class View(FreezableMmifObject):
         else:
             return self.metadata.new_contain(at_type, contain_dict)
 
-    def new_annotation(self, aid: str, at_type: Union[str, ThingTypesBase], overwrite=False) -> 'Annotation':
+    def new_annotation(self, at_type: Union[str, ThingTypesBase], aid: Optional[str] = None, overwrite=False) -> 'Annotation':
         """
         Generates a new :class:`mmif.serialize.annotation.Annotation`
         object and adds it to the current view.
@@ -78,7 +79,14 @@ class View(FreezableMmifObject):
         """
         new_annotation = Annotation()
         new_annotation.at_type = at_type
-        new_annotation.id = aid
+        if aid is not None:
+            new_annotation.id = aid
+        else:
+            prefix = new_annotation.at_type.get_prefix()
+            new_num = self._id_counts.get(prefix, 0) + 1
+            new_id = f'{prefix}_{new_num}'
+            self._id_counts[prefix] = new_num
+            new_annotation.id = new_id
         return self.add_annotation(new_annotation, overwrite)
 
     def add_annotation(self, annotation: 'Annotation', overwrite=False) -> 'Annotation':
@@ -97,6 +105,8 @@ class View(FreezableMmifObject):
                           in the view
         :return: the same Annotation object passed in as ``annotation``
         """
+        if self.is_frozen():
+            raise TypeError("MMIF object is frozen")
         self.annotations.append(annotation, overwrite)
         self.new_contain(annotation.at_type)
         return annotation
@@ -128,8 +138,8 @@ class View(FreezableMmifObject):
             return any(k in prop and prop[k] == v for prop in props)
 
         for annotation in self.annotations:
-            at_type_metadata = self.metadata.contains.get(str(annotation.at_type), {})
-            if not at_type or (at_type and str(annotation.at_type) == str(at_type)):
+            at_type_metadata = self.metadata.contains.get(annotation.at_type, {})
+            if not at_type or (at_type and annotation.at_type == at_type):
                 if all(map(lambda kv: prop_check(kv[0], kv[1], annotation.properties, at_type_metadata), properties.items())):
                     yield annotation
 
@@ -196,24 +206,6 @@ class ViewMetadata(FreezableMmifObject):
         # see MmifObject::_required_attributes in model.py 
         super().__init__(viewmetadata_obj)
 
-    def _find_match_hotfix(self, key: str) -> bool:
-        """
-        Checks the existing types in the contains dict to see if
-        the type passed in as ``key`` has the same shortname.
-
-        FIXME: this will produce undesired results if there is a
-         shortname conflict in the view.
-
-        :param key: the type (shortname or IRI) to check
-        :return: whether ``key`` already has a match in the ``contains`` dict
-        """
-        exists = False
-        for existing_type in self.contains.keys():
-            if key.split('/')[-1] == existing_type.split('/')[-1]:
-                exists = True
-                break
-        return exists
-
     def new_contain(self, at_type: Union[str, ThingTypesBase], contain_dict: dict = None) -> Optional['Contain']:
         """
         Adds a new element to the ``contains`` dictionary.
@@ -222,16 +214,12 @@ class ViewMetadata(FreezableMmifObject):
         :param contain_dict: any metadata associated with the annotation type
         :return: the generated :class:`Contain` object
         """
-        if isinstance(at_type, ThingTypesBase):
-            exists = self._find_match_hotfix(at_type.name) or self._find_match_hotfix(at_type.value)
-            final_key = at_type.value
-        else:
-            exists = self._find_match_hotfix(at_type)
-            final_key = at_type
-
-        if not exists:
+        if isinstance(at_type, str):
+            at_type = ThingTypesBase.from_str(at_type)
+            
+        if at_type not in self.contains:
             new_contain = Contain(contain_dict)
-            self.contains[final_key] = new_contain
+            self.contains[at_type] = new_contain
             return new_contain
 
     def add_parameters(self, param_dict: dict = None, **param_kwargs):
@@ -269,28 +257,8 @@ class Contain(FreezableMmifObject):
     """
     Contain object that represents the metadata of a single
     annotation type in the ``contains`` metadata of a MMIF view.
-
-    :param contain_obj: the metadata that defines this object
     """
 
-    def __init__(self, contain_obj: Union[bytes, str, dict] = None) -> None:
-        # TODO (krim @ 8/19/20): rename `producer` to `app` maybe?
-        self.producer: str = ''
-        self.gen_time: Optional[datetime] = None
-        super().__init__(contain_obj)
-
-    def _deserialize(self, input_dict: dict) -> None:
-        """
-        Extends base ``_deserialize`` method to initialize the
-        ``gen_time`` metadata as a :class:`datetime.datetime` object.
-
-        :param input_dict: the metadata that defines this object
-        :return: None
-        """
-        super()._deserialize(input_dict)
-        if 'gen_time' in self.__dict__ and isinstance(self.gen_time, str):
-            self.gen_time = dateutil.parser.isoparse(self.gen_time)
-
 
 class AnnotationsList(FreezableDataList[Union[Annotation, Document]]):
     """
@@ -330,12 +298,21 @@ class AnnotationsList(FreezableDataList[Union[Annotation, Document]]):
         super()._append_with_key(value.id, value, overwrite)
 
 
-class ContainsDict(FreezableDataDict[Contain]):
-    _items: Dict[str, Contain]
+class ContainsDict(FreezableDataDict[ThingTypesBase, Contain]):
 
     def _deserialize(self, input_dict: dict) -> None:
-        self._items = {key: Contain(value) for key, value in input_dict.items()}
+        self._items = {ThingTypesBase.from_str(key): Contain(value) for key, value in input_dict.items()}
 
     def update(self, other: Union[dict, 'ContainsDict'], overwrite=False):
         for k, v in other.items():
+            if isinstance(k, str):
+                k = ThingTypesBase.from_str(k)
             self._append_with_key(k, v, overwrite=overwrite)
+            
+    def get(self, key: Union[str, ThingTypesBase], default=None):
+        if isinstance(key, str):
+            key = ThingTypesBase.from_str(key)
+        return self._items.get(key, default)
+    
+    def __contains__(self, item):
+        return item in list(self._items.keys())
